@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -217,6 +213,8 @@ def print_version_msg():
               'increase the verbosity. I.e.: -vvvv')
 @click.option('--interpret-escapes', '-e', is_flag=True,
               help='Enable interpretation of backslash escapes')
+@click.option('--interpret-emojis', '-j', is_flag=True,
+              help='Enable interpretation of :emoji: definitions')
 @click.option('--debug', '-D', is_flag=True, help='Debug mode')
 @click.option('--version', '-V', is_flag=True,
               help='Display the apprise version and exit.')
@@ -224,7 +222,8 @@ def print_version_msg():
                 metavar='SERVER_URL [SERVER_URL2 [SERVER_URL3]]',)
 def main(body, title, config, attach, urls, notification_type, theme, tag,
          input_format, dry_run, recursion_depth, verbose, disable_async,
-         details, interpret_escapes, plugin_path, debug, version):
+         details, interpret_escapes, interpret_emojis, plugin_path, debug,
+         version):
     """
     Send a notification to all of the specified servers identified by their
     URLs the content provided within the title, body and notification-type.
@@ -312,6 +311,9 @@ def main(body, title, config, attach, urls, notification_type, theme, tag,
         # Interpret Escapes
         interpret_escapes=interpret_escapes,
 
+        # Interpret Emojis
+        interpret_emojis=None if not interpret_emojis else True,
+
         # Set the theme
         theme=theme,
 
