@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -30,50 +26,6 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-# we mirror our base purely for the ability to reset everything; this
-# is generally only used in testing and should not be used by developers
-# It is also used as a means of preventing a module from being reloaded
-# in the event it already exists
-NOTIFY_MODULE_MAP = {}
-
-# Maintains a mapping of all of the Notification services
-NOTIFY_SCHEMA_MAP = {}
-
-# This contains a mapping of all plugins dynamicaly loaded at runtime from
-# external modules such as the @notify decorator
-#
-# The elements here will be additionally added to the NOTIFY_SCHEMA_MAP if
-# there is no conflict otherwise.
-# The structure looks like the following:
-# Module path, e.g. /usr/share/apprise/plugins/my_notify_hook.py
-# {
-#   'path': path,
-#
-#   'notify': {
-#     'schema': {
-#       'name': 'Custom schema name',
-#       'fn_name': 'name_of_function_decorator_was_found_on',
-#       'url': 'schema://any/additional/info/found/on/url'
-#       'plugin': <CustomNotifyWrapperPlugin>
-#    },
-#     'schema2': {
-#       'name': 'Custom schema name',
-#       'fn_name': 'name_of_function_decorator_was_found_on',
-#       'url': 'schema://any/additional/info/found/on/url'
-#       'plugin': <CustomNotifyWrapperPlugin>
-#    }
-#  }
-#
-# Note: that the <CustomNotifyWrapperPlugin> inherits from
-#       NotifyBase
-NOTIFY_CUSTOM_MODULE_MAP = {}
-
-# Maintains a mapping of all configuration schema's supported
-CONFIG_SCHEMA_MAP = {}
-
-# Maintains a mapping of all attachment schema's supported
-ATTACHMENT_SCHEMA_MAP = {}
-
 
 class NotifyType:
     """
