@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -39,16 +35,24 @@ from .. import plugins
 from .. import common
 from ..AppriseAsset import AppriseAsset
 from ..URLBase import URLBase
+from ..ConfigurationManager import ConfigurationManager
 from ..utils import GET_SCHEMA_RE
 from ..utils import parse_list
 from ..utils import parse_bool
 from ..utils import parse_urls
 from ..utils import cwe312_url
+from ..NotificationManager import NotificationManager
 
 # Test whether token is valid or not
 VALID_TOKEN = re.compile(
     r'(?P<token>[a-z0-9][a-z0-9_]+)', re.I)
 
+# Grant access to our Notification Manager Singleton
+N_MGR = NotificationManager()
+
+# Grant access to our Configuration Manager Singleton
+C_MGR = ConfigurationManager()
+
 
 class ConfigBase(URLBase):
     """
@@ -233,7 +237,7 @@ class ConfigBase(URLBase):
                     schema = schema.group('schema').lower()
 
                     # Some basic validation
-                    if schema not in common.CONFIG_SCHEMA_MAP:
+                    if schema not in C_MGR:
                         ConfigBase.logger.warning(
                             'Unsupported include schema {}.'.format(schema))
                         continue
@@ -244,7 +248,7 @@ class ConfigBase(URLBase):
 
                 # Parse our url details of the server object as dictionary
                 # containing all of the information parsed from our URL
-                results = common.CONFIG_SCHEMA_MAP[schema].parse_url(url)
+                results = C_MGR[schema].parse_url(url)
                 if not results:
                     # Failed to parse the server URL
                     self.logger.warning(
@@ -252,11 +256,10 @@ class ConfigBase(URLBase):
                     continue
 
                 # Handle cross inclusion based on allow_cross_includes rules
-                if (common.CONFIG_SCHEMA_MAP[schema].allow_cross_includes ==
+                if (C_MGR[schema].allow_cross_includes ==
                         common.ContentIncludeMode.STRICT
                         and schema not in self.schemas()
-                        and not self.insecure_includes) or \
-                        common.CONFIG_SCHEMA_MAP[schema] \
+                        and not self.insecure_includes) or C_MGR[schema] \
                         .allow_cross_includes == \
                         common.ContentIncludeMode.NEVER:
 
@@ -284,8 +287,7 @@ class ConfigBase(URLBase):
                 try:
                     # Attempt to create an instance of our plugin using the
                     # parsed URL information
-                    cfg_plugin = \
-                        common.CONFIG_SCHEMA_MAP[results['schema']](**results)
+                    cfg_plugin = C_MGR[results['schema']](**results)
 
                 except Exception as e:
                     # the arguments are invalid or can not be used.
@@ -356,6 +358,81 @@ class ConfigBase(URLBase):
         # missing and/or expired.
         return True
 
+    @staticmethod
+    def __normalize_tag_groups(group_tags):
+        """
+        Used to normalize a tag assign map which looks like:
+          {
+             'group': set('{tag1}', '{group1}', '{tag2}'),
+             'group1': set('{tag2}','{tag3}'),
+          }
+
+          Then normalized it (merging groups); with respect to the above, the
+          output would be:
+          {
+             'group': set('{tag1}', '{tag2}', '{tag3}),
+             'group1': set('{tag2}','{tag3}'),
+          }
+
+        """
+        # Prepare a key set list we can use
+        tag_groups = set([str(x) for x in group_tags.keys()])
+
+        def _expand(tags, ignore=None):
+            """
+            Expands based on tag provided and returns a set
+
+            this also updates the group_tags while it goes
+            """
+
+            # Prepare ourselves a return set
+            results = set()
+            ignore = set() if ignore is None else ignore
+
+            # track groups
+            groups = set()
+
+            for tag in tags:
+                if tag in ignore:
+                    continue
+
+                # Track our groups
+                groups.add(tag)
+
+                # Store what we know is worth keeping
+                if tag not in group_tags:  # pragma: no cover
+                    # handle cases where the tag doesn't exist
+                    group_tags[tag] = set()
+
+                results |= group_tags[tag] - tag_groups
+
+                # Get simple tag assignments
+                found = group_tags[tag] & tag_groups
+                if not found:
+                    continue
+
+                for gtag in found:
+                    if gtag in ignore:
+                        continue
+
+                    # Go deeper (recursion)
+                    ignore.add(tag)
+                    group_tags[gtag] = _expand(set([gtag]), ignore=ignore)
+                    results |= group_tags[gtag]
+
+                    # Pop ignore
+                    ignore.remove(tag)
+
+            return results
+
+        for tag in tag_groups:
+            # Get our tags
+            group_tags[tag] |= _expand(set([tag]))
+            if not group_tags[tag]:
+                ConfigBase.logger.warning(
+                    'The group {} has no tags assigned to it'.format(tag))
+                del group_tags[tag]
+
     @staticmethod
     def parse_url(url, verify_host=True):
         """Parses the URL and returns it broken apart into a dictionary.
@@ -541,6 +618,9 @@ class ConfigBase(URLBase):
             # as additional configuration entries when loaded.
             include <ConfigURL>
 
+            # Assign tag contents to a group identifier
+            <Group(s)>=<Tag(s)>
+
         """
         # A list of loaded Notification Services
         servers = list()
@@ -549,6 +629,12 @@ class ConfigBase(URLBase):
         # the include keyword
         configs = list()
 
+        # Track all of the tags we want to assign later on
+        group_tags = {}
+
+        # Track our entries to preload
+        preloaded = []
+
         # Prepare our Asset Object
         asset = asset if isinstance(asset, AppriseAsset) else AppriseAsset()
 
@@ -556,7 +642,7 @@ class ConfigBase(URLBase):
         valid_line_re = re.compile(
             r'^\s*(?P<line>([;#]+(?P<comment>.*))|'
             r'(\s*(?P<tags>[a-z0-9, \t_-]+)\s*=|=)?\s*'
-            r'(?P<url>[a-z0-9]{2,9}://.*)|'
+            r'((?P<url>[a-z0-9]{1,12}://.*)|(?P<assign>[a-z0-9, \t_-]+))|'
             r'include\s+(?P<config>.+))?\s*$', re.I)
 
         try:
@@ -582,8 +668,13 @@ class ConfigBase(URLBase):
                 # otherwise.
                 return (list(), list())
 
-            url, config = result.group('url'), result.group('config')
-            if not (url or config):
+            # Retrieve our line
+            url, assign, config = \
+                result.group('url'), \
+                result.group('assign'), \
+                result.group('config')
+
+            if not (url or config or assign):
                 # Comment/empty line; do nothing
                 continue
 
@@ -603,6 +694,33 @@ class ConfigBase(URLBase):
             loggable_url = url if not asset.secure_logging \
                 else cwe312_url(url)
 
+            if assign:
+                groups = set(parse_list(result.group('tags'), cast=str))
+                if not groups:
+                    # no tags were assigned
+                    ConfigBase.logger.warning(
+                        'Unparseable tag assignment - no group(s) '
+                        'on line {}'.format(line))
+                    continue
+
+                # Get our tags
+                tags = set(parse_list(assign, cast=str))
+                if not tags:
+                    # no tags were assigned
+                    ConfigBase.logger.warning(
+                        'Unparseable tag assignment - no tag(s) to assign '
+                        'on line {}'.format(line))
+                    continue
+
+                # Update our tag group map
+                for tag_group in groups:
+                    if tag_group not in group_tags:
+                        group_tags[tag_group] = set()
+
+                    # ensure our tag group is never included in the assignment
+                    group_tags[tag_group] |= tags - set([tag_group])
+                continue
+
             # Acquire our url tokens
             results = plugins.url_to_dict(
                 url, secure_logging=asset.secure_logging)
@@ -615,25 +733,56 @@ class ConfigBase(URLBase):
 
             # Build a list of tags to associate with the newly added
             # notifications if any were set
-            results['tag'] = set(parse_list(result.group('tags')))
+            results['tag'] = set(parse_list(result.group('tags'), cast=str))
 
             # Set our Asset Object
             results['asset'] = asset
 
+            # Store our preloaded entries
+            preloaded.append({
+                'results': results,
+                'line': line,
+                'loggable_url': loggable_url,
+            })
+
+        #
+        # Normalize Tag Groups
+        # - Expand Groups of Groups so that they don't exist
+        #
+        ConfigBase.__normalize_tag_groups(group_tags)
+
+        #
+        # URL Processing
+        #
+        for entry in preloaded:
+            # Point to our results entry for easier reference below
+            results = entry['results']
+
+            #
+            # Apply our tag groups if they're defined
+            #
+            for group, tags in group_tags.items():
+                # Detect if anything assigned to this tag also maps back to a
+                # group.  If so we want to add the group to our list
+                if next((True for tag in results['tag']
+                         if tag in tags), False):
+                    results['tag'].add(group)
+
             try:
                 # Attempt to create an instance of our plugin using the
                 # parsed URL information
-                plugin = common.NOTIFY_SCHEMA_MAP[results['schema']](**results)
+                plugin = N_MGR[results['schema']](**results)
 
                 # Create log entry of loaded URL
                 ConfigBase.logger.debug(
-                    'Loaded URL: %s', plugin.url(privacy=asset.secure_logging))
+                    'Loaded URL: %s', plugin.url(
+                        privacy=results['asset'].secure_logging))
 
             except Exception as e:
                 # the arguments are invalid or can not be used.
                 ConfigBase.logger.warning(
                     'Could not load URL {} on line {}.'.format(
-                        loggable_url, line))
+                        entry['loggable_url'], entry['line']))
                 ConfigBase.logger.debug('Loading Exception: %s' % str(e))
                 continue
 
@@ -665,6 +814,12 @@ class ConfigBase(URLBase):
         # the include keyword
         configs = list()
 
+        # Group Assignments
+        group_tags = {}
+
+        # Track our entries to preload
+        preloaded = []
+
         try:
             # Load our data (safely)
             result = yaml.load(content, Loader=yaml.SafeLoader)
@@ -746,9 +901,73 @@ class ConfigBase(URLBase):
         tags = result.get('tag', None)
         if tags and isinstance(tags, (list, tuple, str)):
             # Store any preset tags
-            global_tags = set(parse_list(tags))
+            global_tags = set(parse_list(tags, cast=str))
 
         #
+        # groups root directive
+        #
+        groups = result.get('groups', None)
+        if isinstance(groups, dict):
+            #
+            # Dictionary
+            #
+            for _groups, tags in groups.items():
+                for group in parse_list(_groups, cast=str):
+                    if isinstance(tags, (list, tuple)):
+                        _tags = set()
+                        for e in tags:
+                            if isinstance(e, dict):
+                                _tags |= set(e.keys())
+                            else:
+                                _tags |= set(parse_list(e, cast=str))
+
+                        # Final assignment
+                        tags = _tags
+
+                    else:
+                        tags = set(parse_list(tags, cast=str))
+
+                    if group not in group_tags:
+                        group_tags[group] = tags
+
+                    else:
+                        group_tags[group] |= tags
+
+        elif isinstance(groups, (list, tuple)):
+            #
+            # List of Dictionaries
+            #
+
+            # Iterate over each group defined and store it
+            for no, entry in enumerate(groups):
+                if not isinstance(entry, dict):
+                    ConfigBase.logger.warning(
+                        'No assignment for group {}, entry #{}'.format(
+                            entry, no + 1))
+                    continue
+
+                for _groups, tags in entry.items():
+                    for group in parse_list(_groups, cast=str):
+                        if isinstance(tags, (list, tuple)):
+                            _tags = set()
+                            for e in tags:
+                                if isinstance(e, dict):
+                                    _tags |= set(e.keys())
+                                else:
+                                    _tags |= set(parse_list(e, cast=str))
+
+                            # Final assignment
+                            tags = _tags
+
+                        else:
+                            tags = set(parse_list(tags, cast=str))
+
+                        if group not in group_tags:
+                            group_tags[group] = tags
+
+                        else:
+                            group_tags[group] |= tags
+
         # include root directive
         #
         includes = result.get('include', None)
@@ -880,7 +1099,7 @@ class ConfigBase(URLBase):
                                 del entries['schema']
 
                             # support our special tokens (if they're present)
-                            if schema in common.NOTIFY_SCHEMA_MAP:
+                            if schema in N_MGR:
                                 entries = ConfigBase._special_token_handler(
                                     schema, entries)
 
@@ -892,7 +1111,7 @@ class ConfigBase(URLBase):
 
                 elif isinstance(tokens, dict):
                     # support our special tokens (if they're present)
-                    if schema in common.NOTIFY_SCHEMA_MAP:
+                    if schema in N_MGR:
                         tokens = ConfigBase._special_token_handler(
                             schema, tokens)
 
@@ -926,7 +1145,7 @@ class ConfigBase(URLBase):
                 # Grab our first item
                 _results = results.pop(0)
 
-                if _results['schema'] not in common.NOTIFY_SCHEMA_MAP:
+                if _results['schema'] not in N_MGR:
                     # the arguments are invalid or can not be used.
                     ConfigBase.logger.warning(
                         'An invalid Apprise schema ({}) in YAML configuration '
@@ -938,8 +1157,8 @@ class ConfigBase(URLBase):
                 # The below ensures our tags are set correctly
                 if 'tag' in _results:
                     # Tidy our list up
-                    _results['tag'] = \
-                        set(parse_list(_results['tag'])) | global_tags
+                    _results['tag'] = set(
+                        parse_list(_results['tag'], cast=str)) | global_tags
 
                 else:
                     # Just use the global settings
@@ -965,29 +1184,58 @@ class ConfigBase(URLBase):
                 # Prepare our Asset Object
                 _results['asset'] = asset
 
-                # Now we generate our plugin
-                try:
-                    # Attempt to create an instance of our plugin using the
-                    # parsed URL information
-                    plugin = common.\
-                        NOTIFY_SCHEMA_MAP[_results['schema']](**_results)
+                # Store our preloaded entries
+                preloaded.append({
+                    'results': _results,
+                    'entry': no + 1,
+                    'item': entry,
+                })
 
-                    # Create log entry of loaded URL
-                    ConfigBase.logger.debug(
-                        'Loaded URL: {}'.format(
-                            plugin.url(privacy=asset.secure_logging)))
+        #
+        # Normalize Tag Groups
+        # - Expand Groups of Groups so that they don't exist
+        #
+        ConfigBase.__normalize_tag_groups(group_tags)
 
-                except Exception as e:
-                    # the arguments are invalid or can not be used.
-                    ConfigBase.logger.warning(
-                        'Could not load Apprise YAML configuration '
-                        'entry #{}, item #{}'
-                        .format(no + 1, entry))
-                    ConfigBase.logger.debug('Loading Exception: %s' % str(e))
-                    continue
+        #
+        # URL Processing
+        #
+        for entry in preloaded:
+            # Point to our results entry for easier reference below
+            results = entry['results']
 
-                # if we reach here, we successfully loaded our data
-                servers.append(plugin)
+            #
+            # Apply our tag groups if they're defined
+            #
+            for group, tags in group_tags.items():
+                # Detect if anything assigned to this tag also maps back to a
+                # group.  If so we want to add the group to our list
+                if next((True for tag in results['tag']
+                         if tag in tags), False):
+                    results['tag'].add(group)
+
+            # Now we generate our plugin
+            try:
+                # Attempt to create an instance of our plugin using the
+                # parsed URL information
+                plugin = N_MGR[results['schema']](**results)
+
+                # Create log entry of loaded URL
+                ConfigBase.logger.debug(
+                    'Loaded URL: %s', plugin.url(
+                        privacy=results['asset'].secure_logging))
+
+            except Exception as e:
+                # the arguments are invalid or can not be used.
+                ConfigBase.logger.warning(
+                    'Could not load Apprise YAML configuration '
+                    'entry #{}, item #{}'
+                    .format(entry['entry'], entry['item']))
+                ConfigBase.logger.debug('Loading Exception: %s' % str(e))
+                continue
+
+            # if we reach here, we successfully loaded our data
+            servers.append(plugin)
 
         return (servers, configs)
 
@@ -1023,8 +1271,7 @@ class ConfigBase(URLBase):
         # Create a copy of our dictionary
         tokens = tokens.copy()
 
-        for kw, meta in common.NOTIFY_SCHEMA_MAP[schema]\
-                .template_kwargs.items():
+        for kw, meta in N_MGR[schema].template_kwargs.items():
 
             # Determine our prefix:
             prefix = meta.get('prefix', '+')
@@ -1067,8 +1314,7 @@ class ConfigBase(URLBase):
         #
         # This function here allows these mappings to take place within the
         # YAML file as independant arguments.
-        class_templates = \
-            plugins.details(common.NOTIFY_SCHEMA_MAP[schema])
+        class_templates = plugins.details(N_MGR[schema])
 
         for key in list(tokens.keys()):
 
