@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -64,6 +60,11 @@ from ..AppriseLocale import gettext_lazy as _
 from ..attachment.AttachBase import AttachBase
 
 
+# Used to detect user/role IDs
+USER_ROLE_DETECTION_RE = re.compile(
+    r'\s*(?:<@(?P<role>&?)(?P<id>[0-9]+)>|@(?P<value>[a-z0-9]+))', re.I)
+
+
 class NotifyDiscord(NotifyBase):
     """
     A wrapper to Discord Notifications
@@ -104,6 +105,10 @@ class NotifyDiscord(NotifyBase):
     # The maximum allowable characters allowed in the body per message
     body_maxlen = 2000
 
+    # The 2000 characters above defined by the body_maxlen include that of the
+    # title.  Setting this to True ensures overflow options behave properly
+    overflow_amalgamate_title = True
+
     # Discord has a limit of the number of fields you can include in an
     # embeds message. This value allows the discord message to safely
     # break into multiple messages to handle these cases.
@@ -340,6 +345,33 @@ class NotifyDiscord(NotifyBase):
                 payload['content'] = \
                     body if not title else "{}\r\n{}".format(title, body)
 
+            # parse for user id's <@123> and role IDs <@&456>
+            results = USER_ROLE_DETECTION_RE.findall(body)
+            if results:
+                payload['allow_mentions'] = {
+                    'parse': [],
+                    'users': [],
+                    'roles': [],
+                }
+
+                _content = []
+                for (is_role, no, value) in results:
+                    if value:
+                        payload['allow_mentions']['parse'].append(value)
+                        _content.append(f'@{value}')
+
+                    elif is_role:
+                        payload['allow_mentions']['roles'].append(no)
+                        _content.append(f'<@&{no}>')
+
+                    else:  # is_user
+                        payload['allow_mentions']['users'].append(no)
+                        _content.append(f'<@{no}>')
+
+                if self.notify_format == NotifyFormat.MARKDOWN:
+                    # Add pingable elements to content field
+                    payload['content'] = '👉 ' + ' '.join(_content)
+
             if not self._send(payload, params=params):
                 # We failed to post our message
                 return False
@@ -364,16 +396,21 @@ class NotifyDiscord(NotifyBase):
                 'wait': True,
             })
 
+            #
             # Remove our text/title based content for attachment use
+            #
             if 'embeds' in payload:
-                # Markdown
                 del payload['embeds']
 
             if 'content' in payload:
-                # Markdown
                 del payload['content']
 
+            if 'allow_mentions' in payload:
+                del payload['allow_mentions']
+
+            #
             # Send our attachments
+            #
             for attachment in attach:
                 self.logger.info(
                     'Posting Discord Attachment {}'.format(attachment.name))
