@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -590,7 +586,7 @@ class NotifyMatrix(NotifyBase):
         attachments = None
         if attach and self.attachment_support:
             attachments = self._send_attachments(attach)
-            if not attachments:
+            if attachments is False:
                 # take an early exit
                 return False
 
@@ -615,31 +611,46 @@ class NotifyMatrix(NotifyBase):
                 self.image_url(notify_type)
 
             # Build our path
-            path = '/rooms/{}/send/m.room.message'.format(
-                NotifyMatrix.quote(room_id))
+            if self.version == MatrixVersion.V3:
+                path = '/rooms/{}/send/m.room.message/0'.format(
+                    NotifyMatrix.quote(room_id))
 
-            if image_url:
-                # Define our payload
-                image_payload = {
-                    'msgtype': 'm.image',
-                    'url': image_url,
-                    'body': '{}'.format(notify_type if not title else title),
-                }
+            else:
+                path = '/rooms/{}/send/m.room.message'.format(
+                    NotifyMatrix.quote(room_id))
 
-                # Post our content
-                postokay, response = self._fetch(path, payload=image_payload)
-                if not postokay:
-                    # Mark our failure
-                    has_error = True
-                    continue
+            if self.version == MatrixVersion.V2:
+                #
+                # Attachments don't work beyond V2 at this time
+                #
+                if image_url:
+                    # Define our payload
+                    image_payload = {
+                        'msgtype': 'm.image',
+                        'url': image_url,
+                        'body': '{}'.format(
+                            notify_type if not title else title),
+                    }
 
-            if attachments:
-                for attachment in attachments:
-                    postokay, response = self._fetch(path, payload=attachment)
-                if not postokay:
-                    # Mark our failure
-                    has_error = True
-                    continue
+                    # Post our content
+                    postokay, response = self._fetch(
+                        path, payload=image_payload)
+                    if not postokay:
+                        # Mark our failure
+                        has_error = True
+                        continue
+
+                if attachments:
+                    for attachment in attachments:
+                        attachment['room_id'] = room_id
+                        attachment['type'] = 'm.room.message'
+
+                        postokay, response = self._fetch(
+                            path, payload=attachment)
+                        if not postokay:
+                            # Mark our failure
+                            has_error = True
+                            continue
 
             # Define our payload
             payload = {
@@ -671,7 +682,9 @@ class NotifyMatrix(NotifyBase):
                 })
 
             # Post our content
-            postokay, response = self._fetch(path, payload=payload)
+            method = 'PUT' if self.version == MatrixVersion.V3 else 'POST'
+            postokay, response = self._fetch(
+                path, payload=payload, method=method)
             if not postokay:
                 # Notify our user
                 self.logger.warning(
@@ -689,6 +702,11 @@ class NotifyMatrix(NotifyBase):
         """
 
         payloads = []
+        if self.version != MatrixVersion.V2:
+            self.logger.warning(
+                'Add ?v=2 to Apprise URL to support Attachments')
+            return next((False for a in attach if not a), [])
+
         for attachment in attach:
             if not attachment:
                 # invalid attachment (bad file)
@@ -709,6 +727,29 @@ class NotifyMatrix(NotifyBase):
             #     "content_uri": "mxc://example.com/a-unique-key"
             # }
 
+            # FUTURE if self.version == MatrixVersion.V3:
+            # FUTURE     # Prepare our payload
+            # FUTURE     payloads.append({
+            # FUTURE         "body": attachment.name,
+            # FUTURE         "info": {
+            # FUTURE             "mimetype": attachment.mimetype,
+            # FUTURE             "size": len(attachment),
+            # FUTURE         },
+            # FUTURE         "msgtype": "m.image",
+            # FUTURE         "url": response.get('content_uri'),
+            # FUTURE     })
+
+            # FUTURE else:
+            # FUTURE     # Prepare our payload
+            # FUTURE     payloads.append({
+            # FUTURE         "info": {
+            # FUTURE             "mimetype": attachment.mimetype,
+            # FUTURE         },
+            # FUTURE         "msgtype": "m.image",
+            # FUTURE         "body": "tta.webp",
+            # FUTURE         "url": response.get('content_uri'),
+            # FUTURE     })
+
             # Prepare our payload
             payloads.append({
                 "info": {
@@ -784,12 +825,23 @@ class NotifyMatrix(NotifyBase):
                 'user/pass combo is missing.')
             return False
 
-        # Prepare our Registration Payload
-        payload = {
-            'type': 'm.login.password',
-            'user': self.user,
-            'password': self.password,
-        }
+        # Prepare our Authentication Payload
+        if self.version == MatrixVersion.V3:
+            payload = {
+                'type': 'm.login.password',
+                'identifier': {
+                    'type': 'm.id.user',
+                    'user': self.user,
+                },
+                'password': self.password,
+            }
+
+        else:
+            payload = {
+                'type': 'm.login.password',
+                'user': self.user,
+                'password': self.password,
+            }
 
         # Build our URL
         postokay, response = self._fetch('/login', payload=payload)
@@ -1089,11 +1141,12 @@ class NotifyMatrix(NotifyBase):
                 or self.port == default_port else f':{self.port}')
 
         if path == '/upload':
-            if self.version == MatrixVersion.V3:
-                url += MATRIX_V3_MEDIA_PATH + path
+            # FUTURE if self.version == MatrixVersion.V3:
+            # FUTURE     url += MATRIX_V3_MEDIA_PATH + path
 
-            else:
-                url += MATRIX_V2_MEDIA_PATH + path
+            # FUTURE else:
+            # FUTURE     url += MATRIX_V2_MEDIA_PATH + path
+            url += MATRIX_V2_MEDIA_PATH + path
 
             params = {'filename': attachment.name}
             with open(attachment.path, 'rb') as fp:
@@ -1113,7 +1166,8 @@ class NotifyMatrix(NotifyBase):
         response = {}
 
         # fetch function
-        fn = requests.post if method == 'POST' else requests.get
+        fn = requests.post if method == 'POST' else (
+            requests.put if method == 'PUT' else requests.get)
 
         # Define how many attempts we'll make if we get caught in a throttle
         # event
@@ -1141,7 +1195,9 @@ class NotifyMatrix(NotifyBase):
                     timeout=self.request_timeout,
                 )
 
-                self.logger.debug('Matrix Response: %s' % str(r.content))
+                self.logger.debug(
+                    'Matrix Response: code=%d, %s' % (
+                        r.status_code, str(r.content)))
                 response = loads(r.content)
 
                 if r.status_code == 429:
