@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -118,6 +114,10 @@ class NotifyNextcloud(NotifyBase):
             'min': 1,
             'default': 21,
         },
+        'url_prefix': {
+            'name': _('URL Prefix'),
+            'type': 'string',
+        },
         'to': {
             'alias_of': 'targets',
         },
@@ -131,17 +131,15 @@ class NotifyNextcloud(NotifyBase):
         },
     }
 
-    def __init__(self, targets=None, version=None, headers=None, **kwargs):
+    def __init__(self, targets=None, version=None, headers=None,
+                 url_prefix=None, **kwargs):
         """
         Initialize Nextcloud Object
         """
         super().__init__(**kwargs)
 
+        # Store our targets
         self.targets = parse_list(targets)
-        if len(self.targets) == 0:
-            msg = 'At least one Nextcloud target user must be specified.'
-            self.logger.warning(msg)
-            raise TypeError(msg)
 
         self.version = self.template_args['version']['default']
         if version is not None:
@@ -157,6 +155,10 @@ class NotifyNextcloud(NotifyBase):
                 self.logger.warning(msg)
                 raise TypeError(msg)
 
+        # Support URL Prefix
+        self.url_prefix = '' if not url_prefix \
+            else url_prefix.strip('/')
+
         self.headers = {}
         if headers:
             # Store our extra headers
@@ -169,6 +171,11 @@ class NotifyNextcloud(NotifyBase):
         Perform Nextcloud Notification
         """
 
+        if len(self.targets) == 0:
+            # There were no services to notify
+            self.logger.warning('There were no Nextcloud targets to notify.')
+            return False
+
         # Prepare our Header
         headers = {
             'User-Agent': self.app_id,
@@ -200,11 +207,11 @@ class NotifyNextcloud(NotifyBase):
                 auth = (self.user, self.password)
 
             # Nextcloud URL based on version used
-            notify_url = '{schema}://{host}/ocs/v2.php/'\
+            notify_url = '{schema}://{host}/{url_prefix}/ocs/v2.php/'\
                 'apps/admin_notifications/' \
                 'api/v1/notifications/{target}' \
                 if self.version < 21 else \
-                '{schema}://{host}/ocs/v2.php/'\
+                '{schema}://{host}/{url_prefix}/ocs/v2.php/'\
                 'apps/notifications/'\
                 'api/v2/admin_notifications/{target}'
 
@@ -212,6 +219,7 @@ class NotifyNextcloud(NotifyBase):
                 schema='https' if self.secure else 'http',
                 host=self.host if not isinstance(self.port, int)
                 else '{}:{}'.format(self.host, self.port),
+                url_prefix=self.url_prefix,
                 target=target,
             )
 
@@ -281,6 +289,9 @@ class NotifyNextcloud(NotifyBase):
         # Set our version
         params['version'] = str(self.version)
 
+        if self.url_prefix:
+            params['url_prefix'] = self.url_prefix
+
         # Extend our parameters
         params.update(self.url_parameters(privacy=privacy, *args, **kwargs))
 
@@ -318,7 +329,8 @@ class NotifyNextcloud(NotifyBase):
         """
         Returns the number of targets associated with this notification
         """
-        return len(self.targets)
+        targets = len(self.targets)
+        return targets if targets else 1
 
     @staticmethod
     def parse_url(url):
@@ -347,6 +359,12 @@ class NotifyNextcloud(NotifyBase):
             results['version'] = \
                 NotifyNextcloud.unquote(results['qsd']['version'])
 
+        # Support URL Prefixes
+        if 'url_prefix' in results['qsd'] \
+                and len(results['qsd']['url_prefix']):
+            results['url_prefix'] = \
+                NotifyNextcloud.unquote(results['qsd']['url_prefix'])
+
         # Add our headers that the user can potentially over-ride if they wish
         # to to our returned result set and tidy entries by unquoting them
         results['headers'] = {
