@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -216,9 +212,6 @@ class NotifyXML(NotifyBase):
             # Store our extra headers
             self.headers.update(headers)
 
-        # Set our xsd url
-        self.xsd_url = self.xsd_default_url.format(version=self.xsd_ver)
-
         self.payload_overrides = {}
         self.payload_extras = {}
         if payload:
@@ -240,11 +233,13 @@ class NotifyXML(NotifyBase):
                     self.payload_map[key] = v
                     self.payload_overrides[key] = v
 
-                    # Over-ride XSD URL as data is no longer known
-                    self.xsd_url = None
-
                 else:
                     self.payload_extras[key] = v
+
+        # Set our xsd url
+        self.xsd_url = None if self.payload_overrides or self.payload_extras \
+            else self.xsd_default_url.format(version=self.xsd_ver)
+
         return
 
     def url(self, privacy=False, *args, **kwargs):
