@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -391,9 +387,13 @@ class AppriseURLTester:
                 targets = len(obj)
 
                 # check that we're as expected
-                assert obj.notify(
+                _resp = obj.notify(
                     body=self.body, title=self.title,
-                    notify_type=notify_type) == notify_response
+                    notify_type=notify_type)
+                if _resp != notify_response:
+                    print('%s notify() returned %s (but expected %s)' % (
+                        url, _resp, notify_response))
+                    assert False
 
                 if notify_response:
                     # If we successfully got a response, there must have been
