@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -41,16 +37,21 @@ from apprise import AppriseConfig
 from apprise import AppriseAsset
 from apprise.config.ConfigBase import ConfigBase
 from apprise.plugins.NotifyBase import NotifyBase
+from apprise.NotificationManager import NotificationManager
+from apprise.ConfigurationManager import ConfigurationManager
 
-from apprise.common import CONFIG_SCHEMA_MAP
-from apprise.common import NOTIFY_SCHEMA_MAP
-from apprise.config import __load_matrix
 from apprise.config.ConfigFile import ConfigFile
 
 # Disable logging for a cleaner testing output
 import logging
 logging.disable(logging.CRITICAL)
 
+# Grant access to our Notification Manager Singleton
+N_MGR = NotificationManager()
+
+# Grant access to our Configuration Manager Singleton
+C_MGR = ConfigurationManager()
+
 
 def test_apprise_config(tmpdir):
     """
@@ -252,7 +253,7 @@ def test_apprise_multi_config_entries(tmpdir):
             return ''
 
     # Store our good notification in our schema map
-    NOTIFY_SCHEMA_MAP['good'] = GoodNotification
+    N_MGR._schema_map['good'] = GoodNotification
 
     # Create ourselves a config object
     ac = AppriseConfig()
@@ -472,7 +473,7 @@ def test_apprise_config_instantiate():
             return ConfigBase.parse_url(url, verify_host=False)
 
     # Store our bad configuration in our schema map
-    CONFIG_SCHEMA_MAP['bad'] = BadConfig
+    C_MGR['bad'] = BadConfig
 
     with pytest.raises(TypeError):
         AppriseConfig.instantiate(
@@ -505,7 +506,7 @@ def test_invalid_apprise_config(tmpdir):
             return ConfigBase.parse_url(url, verify_host=False)
 
     # Store our bad configuration in our schema map
-    CONFIG_SCHEMA_MAP['bad'] = BadConfig
+    C_MGR['bad'] = BadConfig
 
     # temporary file to work with
     t = tmpdir.mkdir("apprise-bad-obj").join("invalid")
@@ -570,7 +571,7 @@ def test_apprise_config_with_apprise_obj(tmpdir):
             return ''
 
     # Store our good notification in our schema map
-    NOTIFY_SCHEMA_MAP['good'] = GoodNotification
+    N_MGR._schema_map['good'] = GoodNotification
 
     # Create ourselves a config object
     ac = AppriseConfig(cache=False)
@@ -769,9 +770,9 @@ def test_recursive_config_inclusion(tmpdir):
         allow_cross_includes = ContentIncludeMode.NEVER
 
     # store our entries
-    CONFIG_SCHEMA_MAP['never'] = ConfigCrossPostNever
-    CONFIG_SCHEMA_MAP['strict'] = ConfigCrossPostStrict
-    CONFIG_SCHEMA_MAP['always'] = ConfigCrossPostAlways
+    C_MGR['never'] = ConfigCrossPostNever
+    C_MGR['strict'] = ConfigCrossPostStrict
+    C_MGR['always'] = ConfigCrossPostAlways
 
     # Make our new path valid
     suite = tmpdir.mkdir("apprise_config_recursion")
@@ -978,11 +979,6 @@ def test_apprise_config_matrix_load():
     apprise.config.ConfigDummy3 = ConfigDummy3
     apprise.config.ConfigDummy4 = ConfigDummy4
 
-    __load_matrix()
-
-    # Call it again so we detect our entries already loaded
-    __load_matrix()
-
 
 def test_configmatrix_dynamic_importing(tmpdir):
     """
@@ -1056,8 +1052,6 @@ class ConfigBugger(ConfigBase):
         # always parseable
         return ConfigBase.parse_url(url, verify_host=False)""")
 
-    __load_matrix(path=str(base), name=module_name)
-
 
 @mock.patch('os.path.getsize')
 def test_config_base_parse_inaccessible_text_file(mock_getsize, tmpdir):
