@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -211,11 +207,23 @@ def test_apprise_trans_add():
 
     # This throws internally but we handle it gracefully
     al = AppriseLocale.AppriseLocale()
+    with environ('LANGUAGE', 'LC_ALL', 'LC_CTYPE', 'LANG'):
+        # English is the default/fallback type
+        assert al.add('en') is True
 
-    assert al.add('en') is True
+    al = AppriseLocale.AppriseLocale()
+    with environ('LANGUAGE', 'LC_ALL', 'LC_CTYPE', LANG='C.UTF-8'):
+        # Test English Environment
+        assert al.add('en') is True
 
-    # Double add (copy of above) to access logic that prevents adding it again
-    assert al.add('en') is True
+    al = AppriseLocale.AppriseLocale()
+    with environ('LANGUAGE', 'LC_ALL', 'LC_CTYPE', LANG='en_CA.UTF-8'):
+        # Test English Environment
+        assert al.add('en') is True
+
+        # Double add (copy of above) to access logic that prevents adding it
+        # again
+        assert al.add('en') is True
 
     # Invalid Language
     assert al.add('bad') is False
