@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -36,6 +32,7 @@ from datetime import datetime, timedelta
 from datetime import timezone
 import pytest
 import requests
+from json import loads
 
 from apprise.plugins.NotifyDiscord import NotifyDiscord
 from helpers import AppriseURLTester
@@ -43,6 +40,11 @@ from apprise import Apprise
 from apprise import AppriseAttachment
 from apprise import NotifyType
 from apprise import NotifyFormat
+from apprise.common import OverflowMode
+
+from random import choice
+from string import ascii_uppercase as str_alpha
+from string import digits as str_num
 
 # Disable logging for a cleaner testing output
 import logging
@@ -188,6 +190,113 @@ def test_plugin_discord_urls():
     AppriseURLTester(tests=apprise_url_tests).run_all()
 
 
+@mock.patch('requests.post')
+def test_plugin_discord_notifications(mock_post):
+    """
+    NotifyDiscord() Notifications/Ping Support
+
+    """
+
+    # Initialize some generic (but valid) tokens
+    webhook_id = 'A' * 24
+    webhook_token = 'B' * 64
+
+    # Prepare Mock
+    mock_post.return_value = requests.Request()
+    mock_post.return_value.status_code = requests.codes.ok
+
+    # Test our header parsing when not lead with a header
+    body = """
+    # Heading
+    @everyone and @admin, wake and meet our new user <@123>; <@&456>"
+    """
+
+    results = NotifyDiscord.parse_url(
+        f'discord://{webhook_id}/{webhook_token}/?format=markdown')
+
+    assert isinstance(results, dict)
+    assert results['user'] is None
+    assert results['webhook_id'] == webhook_id
+    assert results['webhook_token'] == webhook_token
+    assert results['password'] is None
+    assert results['port'] is None
+    assert results['host'] == webhook_id
+    assert results['fullpath'] == f'/{webhook_token}/'
+    assert results['path'] == f'/{webhook_token}/'
+    assert results['query'] is None
+    assert results['schema'] == 'discord'
+    assert results['url'] == f'discord://{webhook_id}/{webhook_token}/'
+
+    instance = NotifyDiscord(**results)
+    assert isinstance(instance, NotifyDiscord)
+
+    response = instance.send(body=body)
+    assert response is True
+    assert mock_post.call_count == 1
+
+    details = mock_post.call_args_list[0]
+    assert details[0][0] == \
+        f'https://discord.com/api/webhooks/{webhook_id}/{webhook_token}'
+
+    payload = loads(details[1]['data'])
+
+    assert 'allow_mentions' in payload
+    assert 'users' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['users']) == 1
+    assert '123' in payload['allow_mentions']['users']
+    assert 'roles' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['roles']) == 1
+    assert '456' in payload['allow_mentions']['roles']
+    assert 'parse' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['parse']) == 2
+    assert 'everyone' in payload['allow_mentions']['parse']
+    assert 'admin' in payload['allow_mentions']['parse']
+
+    # Reset our object
+    mock_post.reset_mock()
+
+    results = NotifyDiscord.parse_url(
+        f'discord://{webhook_id}/{webhook_token}/?format=text')
+
+    assert isinstance(results, dict)
+    assert results['user'] is None
+    assert results['webhook_id'] == webhook_id
+    assert results['webhook_token'] == webhook_token
+    assert results['password'] is None
+    assert results['port'] is None
+    assert results['host'] == webhook_id
+    assert results['fullpath'] == f'/{webhook_token}/'
+    assert results['path'] == f'/{webhook_token}/'
+    assert results['query'] is None
+    assert results['schema'] == 'discord'
+    assert results['url'] == f'discord://{webhook_id}/{webhook_token}/'
+
+    instance = NotifyDiscord(**results)
+    assert isinstance(instance, NotifyDiscord)
+
+    response = instance.send(body=body)
+    assert response is True
+    assert mock_post.call_count == 1
+
+    details = mock_post.call_args_list[0]
+    assert details[0][0] == \
+        f'https://discord.com/api/webhooks/{webhook_id}/{webhook_token}'
+
+    payload = loads(details[1]['data'])
+
+    assert 'allow_mentions' in payload
+    assert 'users' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['users']) == 1
+    assert '123' in payload['allow_mentions']['users']
+    assert 'roles' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['roles']) == 1
+    assert '456' in payload['allow_mentions']['roles']
+    assert 'parse' in payload['allow_mentions']
+    assert len(payload['allow_mentions']['parse']) == 2
+    assert 'everyone' in payload['allow_mentions']['parse']
+    assert 'admin' in payload['allow_mentions']['parse']
+
+
 @mock.patch('requests.post')
 def test_plugin_discord_general(mock_post):
     """
@@ -489,6 +598,63 @@ def test_plugin_discord_general(mock_post):
     assert response['params'].get('thread_id') == '12345'
 
 
+@mock.patch('requests.post')
+def test_plugin_discord_overflow(mock_post):
+    """
+    NotifyDiscord() Overflow Checks
+
+    """
+
+    # Initialize some generic (but valid) tokens
+    webhook_id = 'A' * 24
+    webhook_token = 'B' * 64
+
+    # Prepare Mock
+    mock_post.return_value = requests.Request()
+    mock_post.return_value.status_code = requests.codes.ok
+
+    # Some variables we use to control the data we work with
+    body_len = 8000
+    title_len = 1024
+
+    # Number of characters per line
+    row = 24
+
+    # Create a large body and title with random data
+    body = ''.join(choice(str_alpha + str_num + ' ') for _ in range(body_len))
+    body = '\r\n'.join([body[i: i + row] for i in range(0, len(body), row)])
+
+    # Create our title using random data
+    title = ''.join(choice(str_alpha + str_num) for _ in range(title_len))
+
+    results = NotifyDiscord.parse_url(
+        f'discord://{webhook_id}/{webhook_token}/?overflow=split')
+
+    assert isinstance(results, dict)
+    assert results['user'] is None
+    assert results['webhook_id'] == webhook_id
+    assert results['webhook_token'] == webhook_token
+    assert results['password'] is None
+    assert results['port'] is None
+    assert results['host'] == webhook_id
+    assert results['fullpath'] == f'/{webhook_token}/'
+    assert results['path'] == f'/{webhook_token}/'
+    assert results['query'] is None
+    assert results['schema'] == 'discord'
+    assert results['url'] == f'discord://{webhook_id}/{webhook_token}/'
+
+    instance = NotifyDiscord(**results)
+    assert isinstance(instance, NotifyDiscord)
+
+    results = instance._apply_overflow(
+        body, title=title, overflow=OverflowMode.SPLIT)
+
+    # Ensure we never exceed 2000 characters
+    for entry in results:
+        assert len(entry['title']) <= instance.title_maxlen
+        assert len(entry['title']) + len(entry['body']) <= instance.body_maxlen
+
+
 @mock.patch('requests.post')
 def test_plugin_discord_markdown_extra(mock_post):
     """
@@ -568,6 +734,26 @@ def test_plugin_discord_attachments(mock_post):
         'https://discord.com/api/webhooks/{}/{}'.format(
             webhook_id, webhook_token)
 
+    # Reset our object
+    mock_post.reset_mock()
+
+    # Test notifications with mentions and attachments in it
+    assert obj.notify(
+        body='Say hello to <@1234>!', notify_type=NotifyType.INFO,
+        attach=attach) is True
+
+    # Test our call count
+    assert mock_post.call_count == 2
+    assert mock_post.call_args_list[0][0][0] == \
+        'https://discord.com/api/webhooks/{}/{}'.format(
+            webhook_id, webhook_token)
+    assert mock_post.call_args_list[1][0][0] == \
+        'https://discord.com/api/webhooks/{}/{}'.format(
+            webhook_id, webhook_token)
+
+    # Reset our object
+    mock_post.reset_mock()
+
     # An invalid attachment will cause a failure
     path = os.path.join(TEST_VAR_DIR, '/invalid/path/to/an/invalid/file.jpg')
     attach = AppriseAttachment(path)
