@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -124,6 +120,12 @@ TEST_URLS = (
     ('mailtos://user:pass@nuxref.com:567?to=l2g@nuxref.com', {
         'instance': NotifyEmail,
     }),
+    ('mailtos://user:pass@domain.com?user=admin@mail-domain.com', {
+        'instance': NotifyEmail,
+    }),
+    ('mailtos://%20@domain.com?user=admin@mail-domain.com', {
+        'instance': NotifyEmail,
+    }),
     ('mailtos://user:pass@nuxref.com:567/l2g@nuxref.com', {
         'instance': NotifyEmail,
     }),
@@ -1387,6 +1389,61 @@ def test_plugin_email_url_parsing(mock_smtp, mock_smtp_ssl):
     assert pw == 'abc123'
     assert user == 'joe@mydomain.nl'
 
+    mock_smtp.reset_mock()
+    mock_smtp_ssl.reset_mock()
+    response.reset_mock()
+
+    # Issue github.com/caronc/apprise/issue/941
+
+    # mail domain = mail-domain.com
+    # host domain = domain.subdomain.com
+    # PASSWORD needs to be fetched since a user= was provided
+    #  - this is an edge case that is tested here
+    results = NotifyEmail.parse_url(
+        'mailtos://PASSWORD@domain.subdomain.com:587?'
+        'user=admin@mail-domain.com&to=mail@mail-domain.com')
+    assert isinstance(results, dict)
+    # From_Addr could not be detected at this stage, but will be
+    # handled during instantiation
+    assert '' == results['from_addr']
+    assert 'admin@mail-domain.com' == results['user']
+    assert results['port'] == 587
+    assert 'domain.subdomain.com' == results['host']
+    assert 'PASSWORD' == results['password']
+    assert 'mail@mail-domain.com' in results['targets']
+
+    obj = Apprise.instantiate(results, suppress_exceptions=False)
+    assert isinstance(obj, NotifyEmail) is True
+
+    # Not that our from_address takes on 'admin@domain.subdomain.com'
+    assert obj.from_addr == ['Apprise', 'admin@domain.subdomain.com']
+
+    assert mock_smtp.call_count == 0
+    assert mock_smtp_ssl.call_count == 0
+    assert response.starttls.call_count == 0
+    assert obj.notify("test") is True
+    assert mock_smtp.call_count == 1
+    assert response.starttls.call_count == 1
+    assert mock_smtp_ssl.call_count == 0
+    assert response.login.call_count == 1
+    assert response.sendmail.call_count == 1
+    # Store our Sent Arguments
+    # Syntax is:
+    #  sendmail(from_addr, to_addrs, msg, mail_options=(), rcpt_options=())
+    #             [0]        [1]     [2]
+    _from = response.sendmail.call_args[0][0]
+    _to = response.sendmail.call_args[0][1]
+    _msg = response.sendmail.call_args[0][2]
+    assert _from == 'admin@domain.subdomain.com'
+    assert isinstance(_to, list)
+    assert len(_to) == 1
+    assert _to[0] == 'mail@mail-domain.com'
+    assert _msg.split('\n')[-3] == 'test'
+
+    user, pw = response.login.call_args[0]
+    assert user == 'admin@mail-domain.com'
+    assert pw == 'PASSWORD'
+
 
 @mock.patch('smtplib.SMTP_SSL')
 @mock.patch('smtplib.SMTP')
@@ -1527,3 +1584,38 @@ def test_plugin_email_plus_in_toemail(mock_smtp, mock_smtp_ssl):
     assert len(_to) == 1
     assert _to[0] == 'test+notification@gmail.com'
     assert _msg.split('\n')[-3] == 'test'
+
+
+@mock.patch('smtplib.SMTP_SSL')
+@mock.patch('smtplib.SMTP')
+def test_plugin_email_formatting_990(mock_smtp, mock_smtp_ssl):
+    """
+    NotifyEmail() GitHub Issue 990
+    https://github.com/caronc/apprise/issues/990
+    Email formatting not working correctly
+
+    """
+
+    response = mock.Mock()
+    mock_smtp_ssl.return_value = response
+    mock_smtp.return_value = response
+
+    results = NotifyEmail.parse_url(
+        'mailtos://mydomain.com?smtp=mail.local.mydomain.com'
+        '&user=noreply@mydomain.com&pass=mypassword'
+        '&from=noreply@mydomain.com&to=me@mydomain.com&mode=ssl&port=465')
+
+    assert isinstance(results, dict)
+    assert 'noreply@mydomain.com' == results['user']
+    assert 'mydomain.com' == results['host']
+    assert 'mail.local.mydomain.com' == results['smtp_host']
+    assert 'mypassword' == results['password']
+    assert 'ssl' == results['secure_mode']
+    assert '465' == results['port']
+    assert 'me@mydomain.com' in results['targets']
+
+    obj = Apprise.instantiate(results, suppress_exceptions=False)
+    assert isinstance(obj, NotifyEmail) is True
+
+    assert len(obj.targets) == 1
+    assert (False, 'me@mydomain.com') in obj.targets
