@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -47,7 +43,7 @@ logging.disable(logging.CRITICAL)
 
 
 if sys.platform not in ["darwin", "linux"]:
-    pytest.skip("Only makes sense on macOS, but also works on Linux",
+    pytest.skip("Only makes sense on macOS, but testable in Linux",
                 allow_module_level=True)
 
 
@@ -60,8 +56,7 @@ def pretend_macos(mocker):
     mocker.patch("platform.mac_ver", return_value=('10.8', ('', '', ''), ''))
 
     # Reload plugin module, in order to re-run module-level code.
-    current_module = sys.modules[__name__]
-    reload_plugin("NotifyMacOSX", replace_in=current_module)
+    reload_plugin("NotifyMacOSX")
 
 
 @pytest.fixture
@@ -97,6 +92,7 @@ def test_plugin_macosx_general_success(macos_notify_environment):
     NotifyMacOSX() general checks
     """
 
+    # Toggle Enable Flag
     obj = apprise.Apprise.instantiate(
         'macosx://_/?image=True', suppress_exceptions=False)
     assert isinstance(obj, NotifyMacOSX) is True
