@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-# BSD 3-Clause License
+# BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
-# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
+# Copyright (c) 2024, Chris Caron <lead2gold@gmail.com>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -14,10 +14,6 @@
 #    this list of conditions and the following disclaimer in the documentation
 #    and/or other materials provided with the distribution.
 #
-# 3. Neither the name of the copyright holder nor the names of its
-#    contributors may be used to endorse or promote products derived from
-#    this software without specific prior written permission.
-#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -31,7 +27,8 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 from unittest import mock
-
+from apprise import Apprise
+from apprise import NotifyType
 import requests
 from apprise.plugins.NotifyNextcloudTalk import NotifyNextcloudTalk
 from helpers import AppriseURLTester
@@ -42,7 +39,7 @@ logging.disable(logging.CRITICAL)
 
 apprise_url_tests = (
     ##################################
-    # NotifyNextcloud
+    # NotifyNextcloudTalk
     ##################################
     ('nctalk://:@/', {
         'instance': None,
@@ -68,7 +65,10 @@ apprise_url_tests = (
     }),
     ('nctalk://user:pass@localhost', {
         # No roomid specified
-        'instance': TypeError,
+        'instance': NotifyNextcloudTalk,
+        # Since there are no targets specified we expect a False return on
+        # send()
+        'notify_response': False,
     }),
     ('nctalk://user:pass@localhost/roomid1/roomid2', {
         'instance': NotifyNextcloudTalk,
@@ -81,6 +81,10 @@ apprise_url_tests = (
         'instance': NotifyNextcloudTalk,
         'requests_response_code': requests.codes.created,
     }),
+    ('nctalk://user:pass@localhost:8080/roomid?url_prefix=/prefix', {
+        'instance': NotifyNextcloudTalk,
+        'requests_response_code': requests.codes.created,
+    }),
     ('nctalks://user:pass@localhost/roomid', {
         'instance': NotifyNextcloudTalk,
         'requests_response_code': requests.codes.created,
@@ -119,7 +123,7 @@ apprise_url_tests = (
 
 def test_plugin_nextcloudtalk_urls():
     """
-    NotifyNextcloud() Apprise URLs
+    NotifyNextcloudTalk() Apprise URLs
 
     """
 
@@ -130,7 +134,7 @@ def test_plugin_nextcloudtalk_urls():
 @mock.patch('requests.post')
 def test_plugin_nextcloudtalk_edge_cases(mock_post):
     """
-    NotifyNextcloud() Edge Cases
+    NotifyNextcloudTalk() Edge Cases
 
     """
 
@@ -152,3 +156,45 @@ def test_plugin_nextcloudtalk_edge_cases(mock_post):
     assert obj.send(body="") is True
     assert 'data' in mock_post.call_args_list[0][1]
     assert 'message' in mock_post.call_args_list[0][1]['data']
+
+
+@mock.patch('requests.post')
+def test_plugin_nextcloud_talk_url_prefix(mock_post):
+    """
+    NotifyNextcloudTalk() URL Prefix Testing
+    """
+
+    response = mock.Mock()
+    response.content = ''
+    response.status_code = requests.codes.created
+
+    # Prepare our mock object
+    mock_post.return_value = response
+
+    # instantiate our object (without a batch mode)
+    obj = Apprise.instantiate(
+        'nctalk://user:pass@localhost/admin/?url_prefix=/abcd')
+
+    assert obj.notify(
+        body='body', title='title', notify_type=NotifyType.INFO) is True
+
+    # Not set to batch, so we send 2 different messages
+    assert mock_post.call_count == 1
+    assert mock_post.call_args_list[0][0][0] == \
+        'http://localhost/abcd/ocs/v2.php/apps/spreed/api/v1/chat/admin'
+
+    mock_post.reset_mock()
+
+    # instantiate our object (without a batch mode)
+    obj = Apprise.instantiate(
+        'nctalk://user:pass@localhost/admin/?'
+        'url_prefix=a/longer/path/abcd/')
+
+    assert obj.notify(
+        body='body', title='title', notify_type=NotifyType.INFO) is True
+
+    # Not set to batch, so we send 2 different messages
+    assert mock_post.call_count == 1
+    assert mock_post.call_args_list[0][0][0] == \
+        'http://localhost/a/longer/path/abcd/' \
+        'ocs/v2.php/apps/spreed/api/v1/chat/admin'
